// RocketFormView.cpp : implementation file
//

#include "stdafx.h"
#include "Rocket.h"
#include "RocketFormView.h"
#include "Progdlg.h"
#include <math.h>
#include <stdio.h>

#ifdef _DEBUG
	#define new DEBUG_NEW
	#undef THIS_FILE
	static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CRocketFormView

IMPLEMENT_DYNCREATE(CRocketFormView, CFormView)

CRocketFormView::CRocketFormView()
	: CFormView(CRocketFormView::IDD)
{
	//{{AFX_DATA_INIT(CRocketFormView)
	m_CD = 0.25;
	m_Density = 1.20014639832;
	m_Engine = 0;
	m_Mass = 50.0;
	m_Pressure = 1013.25;
	m_Radius = 1.195;
	m_Temp = 20.0;
	m_TInc = 0.01;
	m_TEnd = 3.24;
	m_TStart = 0.01;
	m_EndCond = 0;
	m_Accuracy = 0.001;
	//}}AFX_DATA_INIT
	m_TBurn = 0.24;
	m_MassF = 0.00312;
}

CRocketFormView::~CRocketFormView()
{
}

void CRocketFormView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CRocketFormView)
	DDX_Control(pDX, IDC_GO, m_Go);
	DDX_Control(pDX, IDC_OUTPUT, m_ListCtrl);
	DDX_Text(pDX, IDC_CD, m_CD);
	DDV_MinMaxDouble(pDX, m_CD, 0., 0.5);
	DDX_Text(pDX, IDC_DENSITY, m_Density);
	DDX_CBIndex(pDX, IDC_ENGINE, m_Engine);
	DDX_Text(pDX, IDC_MASS, m_Mass);
	DDV_MinMaxDouble(pDX, m_Mass, 25., 500.);
	DDX_Text(pDX, IDC_PRESSURE, m_Pressure);
	DDV_MinMaxDouble(pDX, m_Pressure, 913., 1113.5);
	DDX_Text(pDX, IDC_RADIUS, m_Radius);
	DDV_MinMaxDouble(pDX, m_Radius, 0., 2.5);
	DDX_Text(pDX, IDC_TEMP, m_Temp);
	DDV_MinMaxDouble(pDX, m_Temp, 0., 40.);
	DDX_Text(pDX, IDC_TINC, m_TInc);
	DDV_MinMaxDouble(pDX, m_TInc, 0., 0.1);
	DDX_Text(pDX, IDC_TSTART, m_TStart);
	DDX_CBIndex(pDX, IDC_ENDCOND, m_EndCond);
	DDX_Text(pDX, IDC_ACCURACY, m_Accuracy);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CRocketFormView, CFormView)
	ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCopyCut)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CUT, OnUpdateEditCopyCut)
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
	ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateEditUndo)
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_COMMAND(ID_EDIT_CUT, OnEditCut)
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
	//{{AFX_MSG_MAP(CRocketFormView)
	ON_CBN_CLOSEUP(IDC_ENGINE, OnChangeEngine)
	ON_BN_CLICKED(IDC_GO, OnGo)
	ON_COMMAND(ID_FILE_PRINT, OnFilePrint)
	ON_EN_KILLFOCUS(IDC_MASS, CheckInputValidity)
	ON_EN_KILLFOCUS(IDC_PRESSURE, GetDensity)
	ON_EN_KILLFOCUS(IDC_RADIUS, CheckInputValidity)
	ON_EN_KILLFOCUS(IDC_TINC, CheckInputValidity)
	ON_EN_KILLFOCUS(IDC_CD, CheckInputValidity)
	ON_EN_KILLFOCUS(IDC_TEMP, GetDensity)
	ON_EN_KILLFOCUS(IDC_ACCURACY, CheckInputValidity)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRocketFormView diagnostics

#ifdef _DEBUG
	void CRocketFormView::AssertValid() const
	{
		CFormView::AssertValid();
	}
	
	void CRocketFormView::Dump(CDumpContext& dc) const
	{
		CFormView::Dump(dc);
	}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CRocketFormView message handlers

/////////////////////////////////////////////////////////////////////////////
// Initialization

void CRocketFormView::OnInitialUpdate() 
{
	//Application init
	TRACE0("CRocketFormView::OnInitialUpdate()\n");
	CFormView::OnInitialUpdate(); //Base class call
	ResizeParentToFit();      //
	ResizeParentToFit(FALSE); //Size the application to the dialog				
	Init();
}

void CRocketFormView::Init()
{
	//List control init
	TRACE0("CRocketFormView::Init()\n");			
	while(m_ListCtrl.DeleteColumn(0));
	m_ListCtrl.DeleteAllItems();         //Clear the list box
	char* arColumns[] = {"Time (s)", "Thrust (N)", "Mass (kg)", "Weight (N)", "Acceleration (m/s2)", "Velocity (m/s)", "Drag (N)", "Height (m)"}; //Column labels	
	int arWidths[] = {55, 65, 65, 70, 115, 85, 60, 65};
	LV_COLUMN ListColumn;
	ListColumn.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM; //Valid structure members
	ListColumn.fmt = LVCFMT_CENTER; //Set the text aligment	
	for(int i = 0;i < 8;i++)
	{
		ListColumn.iSubItem = i; //Set the subitem associated with the column
		ListColumn.pszText = arColumns[i]; //Get column text from array arColumns
		ListColumn.cx = arWidths[i]; //Set the width per column
		m_ListCtrl.InsertColumn (i, &ListColumn); //Add the list box header column
	}	
}

/////////////////////////////////////////////////////////////////////////////
// Clipboard handlers

BOOL CRocketFormView::IsEditCtrl(CWnd* pWnd)
{
	// CG: This function was added by the Clipboard Assistant component
	ASSERT_VALID(pWnd);

	TCHAR lpClassName[32];
	::GetClassName(pWnd->GetSafeHwnd(), lpClassName, 32);

	if (!_tcscmp(lpClassName, _T("Edit")))
		return TRUE;

	return FALSE;
}

void CRocketFormView::OnUpdateEditCopyCut(CCmdUI* pCmdUI)
{
	// CG: This block was added by the Clipboard Assistant component
	{
		CWnd* pWnd = CWnd::GetFocus();

		if (pWnd != NULL)
		{
			if (IsEditCtrl(pWnd))
			{
				int nStart, nEnd;
				((CEdit*)pWnd)->GetSel(nStart, nEnd);
				pCmdUI->Enable(nStart != nEnd);
				return;
			}
		}
	}

	pCmdUI->Enable(FALSE);
}

void CRocketFormView::OnUpdateEditPaste(CCmdUI* pCmdUI)
{
	// CG: This block was added by the Clipboard Assistant component
	{
		CWnd* pWnd = CWnd::GetFocus();

		if (pWnd != NULL)
		{
			if (IsEditCtrl(pWnd))
			{
				pCmdUI->Enable(::IsClipboardFormatAvailable(CF_TEXT));
				return;
			}
		}
	}

	pCmdUI->Enable(FALSE);
}

void CRocketFormView::OnUpdateEditUndo(CCmdUI* pCmdUI)
{
	// CG: This block was added by the Clipboard Assistant component
	{
		CWnd* pWnd = CWnd::GetFocus();

		if (pWnd != NULL)
		{
			if (IsEditCtrl(pWnd))
			{
				pCmdUI->Enable(((CEdit*)pWnd)->CanUndo());
				return;
			}
		}
	}
	pCmdUI->Enable(FALSE);
}

void CRocketFormView::OnEditCopy()
{
	// CG: This block was added by the Clipboard Assistant component
	{
		CEdit* pEdit = (CEdit*)CWnd::GetFocus();
		pEdit->Copy();
	}
}

void CRocketFormView::OnEditCut()
{
	// CG: This block was added by the Clipboard Assistant component
	{
		CEdit* pEdit = (CEdit*)CWnd::GetFocus();
		pEdit->Cut();
	}
}

void CRocketFormView::OnEditPaste()
{
	// CG: This block was added by the Clipboard Assistant component
	{
		CEdit* pEdit = (CEdit*)CWnd::GetFocus();
		pEdit->Paste();
	}
}

void CRocketFormView::OnEditUndo()
{
	// CG: This block was added by the Clipboard Assistant component
	{
		CEdit* pEdit = (CEdit*)CWnd::GetFocus();
		pEdit->Undo();
	}
}

/////////////////////////////////////////////////////////////////////////////
// Printing

void CRocketFormView::OnFilePrint() 
{
		
}

BOOL CRocketFormView::OnPreparePrinting(CPrintInfo* pInfo) 
{
	return CFormView::OnPreparePrinting(pInfo);
}

void CRocketFormView::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo) 
{
	m_Font = new CFont;
	LOGFONT lf;
	memset(&lf, 0, sizeof(lf));
	lstrcpy(lf.lfFaceName, "Arial");
	m_Font->CreateFontIndirect(&lf);
	
	CFormView::OnBeginPrinting(pDC, pInfo);
}

void CRocketFormView::OnPrint(CDC* pDC, CPrintInfo* pInfo) 
{	
	CString Text;
	CPoint pt(0, 0);
	int nOldMode = pDC->SetMapMode(MM_TWIPS);
	CFont* pOldFont = pDC->SelectObject(m_Font);
	TEXTMETRIC tm;	
	pDC->GetTextMetrics(&tm);
	int cyText = tm.tmHeight + tm.tmExternalLeading;
	int nLength = (int)(pDC->GetDeviceCaps(VERTSIZE) / 25.4) * -1440;
	for(pt.y -= cyText;pt.y > nLength; pt.y -= cyText)
	{
		pDC->TextOut(pt.x, pt.y, Text);
	}
	pDC->SelectObject(pOldFont);
	pDC->SetMapMode(nOldMode);
}

void CRocketFormView::OnEndPrinting(CDC* pDC, CPrintInfo* pInfo) 
{
	delete m_Font;
	CFormView::OnEndPrinting(pDC, pInfo);
}

/////////////////////////////////////////////////////////////////////////////
// User input

void CRocketFormView::OnChangeEngine() 
{
	//Change engine specific rocket properties
	TRACE0("CRocketFormView::OnChangeEngine()\n");
	UpdateData();
	switch(m_Engine)
	{
		case 0: 
		{
			m_TEnd = 3.24;
			m_TBurn = 0.24;
			m_MassF = 0.00312;
			break;
		}
		case 1: 
		{
			m_TEnd = 4.67;
			m_TBurn = 0.67;
			m_MassF = 0.00624;
			break;
		}
		case 2: 
		{
			m_TEnd = 6.7;
			m_TBurn = 1.7;
			m_MassF = 0.0127;
			break;
		}		
	}	
	UpdateData(FALSE);			
}

void CRocketFormView::GetDensity()
{	
	//Calculate density from temp and pressure
	TRACE0("CRocketFormView::GetDensity()\n");	
	UpdateData();	
	double Temp = m_Temp + 273.15; //Convert temp (c) to k
	m_Density = (m_Pressure / (2.88 * Temp));
	UpdateData(FALSE);
}

void CRocketFormView::CheckInputValidity() 
{	
	//Get input
	TRACE0("CRocketFormView::CheckInputValidity()\n");	
	UpdateData();
}

/////////////////////////////////////////////////////////////////////////////
// Calculation

double CRocketFormView::GetThrust(double t)
{
	//Calculate the thrust of the engine as a function of the time
	TRACE0("CRocketFormView::GetThrust()\n");
	
	//Thrust variable
	double Thrust;
	
	//Select thrust curve based on engine
	switch (m_Engine)
	{
		case 0:
		{
			if(t <= .04) Thrust = 75 * t;
			else if(t <= .08) Thrust = 150 * t - 3;
			else if(t <= .16) Thrust = -2028.708 * pow(t,2) + 508.999 * t - 18.728;
			else if(t <= .22) Thrust = -156 * t + 35.72;
			else Thrust = - 70 * t + 16.8;			
			break;
		}
		case 1:
		{
			if(t <= .12) Thrust = 50 * t;
			else if(t <= .28) Thrust = 156.25 * t - 12.75;
			else if(t <= .34) Thrust = -(1150 / 3) * t + 367 / 3;
			else if(t <= .4) Thrust = -(100 / 3) * t + 22 / 3;
			else if(t <= .8) Thrust = 6;
			else Thrust = -150 * t + 126;					
			break;
		}
		case 2:
		{
			if(t <= .24) Thrust = 325 * t / 6;
			else if(t <= .4) Thrust = -43.75 * t - 23.5;
			else if(t <= 1.68) Thrust = 6;
			else Thrust = -30 * (10 * t - 17);												
		}
	}
	ASSERT(Thrust > 0); //Verify that the thrust is positive	
	return Thrust;
}

void CRocketFormView::OnGo() 
{				
	//Main function
	TRACE0("CRocketFormView::OnGo()\n");
	
	UpdateData(); //Get dialog input	

	//Constants
	const double G = 9.80665; //Acceleration of gravity		 
	const double Area = 3.1415926535898 * pow(m_Radius, 2) * 1E-4; //Area = Pi * Radius(m) ^ 2	
		
	//Variables
	double t = 0, D = 0, Thrust, W, A, A2, V, V2, S, S2, Numbers[8];
	double M = m_Mass * 1E-3; //Convert mass (g) to kg	
	int ActualPos = 0, Pos = 0;
	char buffer[] = "", bigbuffer[100] = "";
		
	//List view init
	Init();
	m_ListCtrl.SetItemCount((int)(m_TEnd / m_TInc));
	LV_ITEM ListItem;
	ListItem.mask = LVIF_TEXT;
		
	//Progress dialog init
	CProgressDlg dlg;
	dlg.Create();
	dlg.SetStatus("Calculating... please wait");

	//Main loop	
	for(t = m_TInc;(m_EndCond == 0) ? t < m_TEnd : V > 0;t += m_TInc)
	{					
		Numbers[0] = t;
		TRACE2("t = %G, Pos = %d \n", t, Pos); //Dump of time and loop increment		

		//Thrust
		Thrust = (t < m_TBurn)? GetThrust(t) : 0; //Calculate only during burn time		
		Numbers[1] = Thrust;
		TRACE1("Thrust = %f\n", Thrust); //Dump of thrust
		
		//Mass, Weight
		if (t < m_TBurn) M -= (m_MassF / m_TBurn) * m_TInc; //While the engine is burning, subtract used fuel from the mass
		W = M * G; //Calculate the current weight of the rocket
		Numbers[2] = M;
		Numbers[3] = W;
		
		//Set last values
		A2 = A;
		V2 = V;
		S2 = S;

		//Acceleration, Velocity, Drag
		do
		{
			A = (Thrust - W - D) / M; //Equation {E}F = M * A solved for A
			V = (A + A2) / 2 * m_TInc + V2; //Integrate V(T) = A * T
			D = .5 * m_CD * m_Density * Area * pow(V, 2); //Calculate the drag
			A2 = (Thrust - W - D) / M; //Acceleration from new values
		}
		while(fabs(A - A2) > m_Accuracy); //Compare A and A2, and keep looping while they are significantly different
		Numbers[4] = A;
		Numbers[5] = V;		
		Numbers[6] = D;
		
		//Altitude
		S = (V + V2) / 2 * m_TInc + S2; //Integrate S(T) = V * T
		ASSERT(S > 0); //Verify that the altitude is positive
		Numbers[7] = S;		
												
		//Display list control output
		for (int i = 0;i < 8;i++)
		{
			ListItem.iItem = (i)? ActualPos : Pos++; //Select the correct list position
			ListItem.iSubItem = i; //Set the column
			sprintf(buffer,"%f",Numbers[i]); //Convert the numerical value
			ListItem.pszText = buffer; //Set the list item text
			if(i) m_ListCtrl.SetItem(&ListItem); //Change the item
			else ActualPos = m_ListCtrl.InsertItem(&ListItem); //Insert the item
		}
		
		//Show calculation status
		dlg.SetPos((int)(t / m_TEnd * 100));
		if(dlg.CheckCancelButton())
		{			
			dlg.DestroyWindow();
			MessageBox("Calculation aborted!", &"Calculation Results", MB_OK | MB_ICONINFORMATION);
			return;
		}
	}	

	//Final output	
	dlg.DestroyWindow();
	sprintf(bigbuffer,"The final altitude of the rocket is %f meters.", Numbers[7]);
	MessageBox(bigbuffer, &"Calculation Results", MB_OK | MB_ICONINFORMATION);
}
